#import "RegistrationPane.h"
#import <stdio.h>
/*
	Important:	(1)	Remember to always compile your plugins as a universal binary.
				(2)	Also be aware that informed end-users can remove your package's Plugins directory and/or
					change your package's InstallerSections.plist file to remove custom plugins from its flow.
					So plugins *cannot* serve as effective gatekeepers to prevent installation.
*/

@implementation RegistrationPane

#pragma mark private methods
/* test all textfields to if they all have at least one character in them */
- (BOOL)_entriesAreValid
{
	if (	([[uiFirstNameField stringValue] length] == 0) ||
			([[uiLastNameField stringValue] length] == 0) ||
			([[uiOrganizationField stringValue] length] == 0) ||
			([[uiSerialNumberField stringValue] length] == 0) )
	{
		return NO;
	}
	
	return YES;
}

/* perform a simple string compare to validate the serial number entered by the user */
- (BOOL)_serialNumberIsValid
{
	// need to check that the val is an integer
	// return [[uiSerialNumberField stringValue] isNumber];
	return YES;
}

/* enable the "Continue" button if "_entriesAreValid" returns 'YES' */
- (void)_updateNextButtonState
{
	[self setNextEnabled:[self _entriesAreValid]];
}

/* localization helper method:  This pulls localized strings from the plugin's bundle */
- (NSString *)_localizedStringForKey:(NSString *)key
{
	return [[NSBundle bundleForClass:[self class]] localizedStringForKey:key value:@"" table:nil];
}

/////////////////////////////////////////////////
#pragma mark InstallerPane overrides
/* return the title of this pane */
- (NSString *)title
{
	return [self _localizedStringForKey:@"Title"];
}

/* pane's entry point: code called when user enters this pane */
- (void)didEnterPane:(InstallerSectionDirection)dir
{
	[self _updateNextButtonState];
}

/* called when user clicks "Continue" -- return value indicates if application should exit pane */
- (BOOL)shouldExitPane:(InstallerSectionDirection)dir
{
	if((InstallerDirectionForward == dir) && ([self _serialNumberIsValid] == NO)) {
		[self _updateNextButtonState];
		
		NSBeginInformationalAlertSheet(nil, [self _localizedStringForKey:@"OK_BUTTON"], nil, nil, [uiFirstNameField window], self, nil, nil, NULL, [self _localizedStringForKey:@"InvalidSerialNumberAlertMessage"]);
		return NO;
	}
	//if ([[[self section] installerState] installStarted] && [[[self section] installerState] installSucceeded]) {
    // [[[self section] installerState] targetPath]] : '/' or '/Users/ivan'
		
	char *java_info_plist = "<?xml version=\"1.0\" encoding=\"UTF-8\"?> \n \
<!DOCTYPE plist SYSTEM \"file://localhost/System/Library/DTDs/PropertyList.dtd\"> \n \
<plist version=\"0.9\"> \n \
<dict> \n \
        <key>CFBundleName</key> \n \
        <string>SatelliteLab</string> \n \
        <key>CFBundleVersion</key> \n \
        <string>100.0</string> \n \
        <key>CFBundleAllowMixedLocalizations</key> \n \
        <string>true</string> \n \
        <key>CFBundleExecutable</key> \n \
        <string>JavaApplicationStub</string> \n \
        <key>CFBundleDevelopmentRegion</key> \n \
        <string>English</string> \n \
        <key>CFBundlePackageType</key> \n \
        <string>APPL</string> \n \
        <key>CFBundleSignature</key> \n \
        <string>????</string> \n \
        <key>CFBundleInfoDictionaryVersion</key> \n \
        <string>6.0</string> \n \
        <key>CFBundleIconFile</key> \n \
        <string>SatelliteLab.ico</string> \n \
        <key>Java</key> \n \
        <dict> \n \
                <key>WorkingDirectory</key> \n \
                <string>$APP_PACKAGE/Contents/Resources/Java</string> \n \
                <key>MainClass</key> \n \
                <string>Satellite</string> \n \
                <key>JVMVersion</key> \n \
                <string>1.5*</string> \n \
                <key>Arguments</key> \n \
                <string>%s %s %s %s</string> \n \
                <key>ClassPath</key> \n \
                        <string>$JAVAROOT/Satellite.jar</string> \n \	
        </dict> \n \
</dict> \n \
</plist> \n";
	FILE* f = fopen("/tmp/tmp_java_info.plist", "w");
	fprintf(f, java_info_plist,
			[[uiFirstNameField stringValue] UTF8String],
			[[uiLastNameField stringValue] UTF8String],
			[[uiOrganizationField stringValue] UTF8String],
			[[uiSerialNumberField stringValue] UTF8String]);
	fclose(f);
		
//		NSBeginInformationalAlertSheet(nil, [self _localizedStringForKey:@"OK_BUTTON"], nil, nil, [uiFirstNameField window], self, nil, nil, NULL, @"install failed"); // '/' or '/Users/ivan'
	return YES;
}

/////////////////////////////////////////////////
#pragma mark NSControl delegate methods
/* updates the state of the next button when the contents of the delegate textfields change */
- (void)controlTextDidChange:(NSNotification *)notification
{
	[self _updateNextButtonState];
}

- (IBAction) uiCheck:(id) sender;
{
	if([uiCheck state]==NSOnState) {
		[uiOrganizationField setEnabled: NO];
		[uiSerialNumberField setEnabled: NO];
	} else {
		[uiOrganizationField setEnabled: YES];
		[uiSerialNumberField setEnabled: YES];
	}
}

@end
