#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <errno.h>
#include <netdb.h>
#include <time.h>
#include <poll.h>
#ifdef __linux__
#include <sys/ioctl.h>
#endif

#define PROBE_HISTORY_LENGTH 100
#define PROBE_HISTORY_MAX_INCLUDE 20
#define SUN_PASSWORD_VALID_MILLIS 120000LL

#define dprintf(a...) do { } while (0)
#define panic(a...) do { fprintf(stderr, a); fprintf(stderr, "\n"); exit(1); } while (0)
#define min(a,b) (((a)<(b)) ? (a) : (b))
#define max(a,b) (((a)>(b)) ? (a) : (b))

struct sockaddr_in planet;
struct sockaddr_in sun;
char email[200];
char identifier[200];
int sunIntervalSec = 59;
int planetIntervalSec = 1;
unsigned char sunPassword[8];
unsigned char planetPassword[100];
long long sunPwdLastChosen;
int planetPasswordLen = 0;
unsigned int totalBytesSent = 0;
unsigned int totalBytesReceived = 0;

long long getTimeMillis()
{
  struct timeval tv;
  gettimeofday(&tv, NULL);
  return tv.tv_sec * 1000LL + (tv.tv_usec/1000);
}

int main(int argc, char *argv[])
{
  int i;

  if (argc != 5) {
    fprintf(stderr, "Syntax: %s <email> <identifier> <sunAddress> <sunPort>\n", argv[0]);
    return 1;
  }
  
  /* Init */
  
  struct hostent *ent = gethostbyname(argv[3]);
  if (!ent) {
    fprintf(stderr, "Cannot resolve host name '%s' (%s)\n", argv[3], strerror(errno));
    return 2;
  }

  int sock = socket(PF_INET, SOCK_DGRAM, 0);
  struct sockaddr_in localAddr;
  memset(&localAddr, 0, sizeof(localAddr));
  localAddr.sin_family = AF_INET;
  localAddr.sin_port = 0;
  localAddr.sin_addr.s_addr = INADDR_ANY;
  if (bind(sock, (struct sockaddr*)&localAddr, sizeof(localAddr)) < 0) {
    fprintf(stderr, "Cannot bind to a local port (%s)\n", strerror(errno));
    return 3;
  }
  
  srandom(time(0));
  
  sun.sin_family = AF_INET;
  sun.sin_port = htons(atoi(argv[4]));
  sun.sin_addr.s_addr = *(in_addr_t*)ent->h_addr;;
    
  planet.sin_family = AF_INET;
  planet.sin_port = 0;
  planet.sin_addr.s_addr = 0;
  
  sunPwdLastChosen = getTimeMillis();
  for (i=0; i<(int)sizeof(sunPassword); i++)
    sunPassword[i] = random() & 0xFF;
    
  strncpy(email, argv[1], sizeof(email));
  strncpy(identifier, argv[2], sizeof(identifier));

  int probeIdsInHistory = 0;
  int probeHistory[PROBE_HISTORY_LENGTH];
  for (i=0; i<PROBE_HISTORY_LENGTH; i++)
    probeHistory[i] = 0;

  /* Become a daemon */

  pid_t pid = fork();
  if (pid != 0)
    exit(0);
  
  close(STDIN_FILENO);
  close(STDOUT_FILENO);
  close(STDERR_FILENO);
  
  setsid();
  signal(SIGHUP, SIG_IGN);
  if ((pid = fork()) != 0)
    exit(0);

  /* The inner loop. This is the code the user needs to trust. */
  
  unsigned char outbuf[65536];
  unsigned char inbuf[65536];
  struct sockaddr_in srcaddr;
  long long nextPlanetReportMillis = getTimeMillis();
  long long nextSunReportMillis = getTimeMillis();

  while (1) {
    long long now = getTimeMillis();
    
    if (now >= nextPlanetReportMillis) {
      nextPlanetReportMillis = now + planetIntervalSec * 1000LL;
      if (planet.sin_addr.s_addr && planet.sin_port) {
        memcpy(&outbuf[0], (unsigned char[]){ 0xFF, 0xFE, 0x53, 0x41, 0x54, 0x4C, 0x01, 0x01 }, 8);
        memcpy(&outbuf[8], planetPassword, planetPasswordLen);
        totalBytesSent += 28+8+planetPasswordLen;
        sendto(sock, &outbuf, 8+planetPasswordLen, 0, (struct sockaddr*)&planet, sizeof(planet));
      }
    }
      
    if (now >= nextSunReportMillis) {
      nextSunReportMillis = now + sunIntervalSec * 1000LL;
      if (now > (sunPwdLastChosen + SUN_PASSWORD_VALID_MILLIS)) {
        sunPwdLastChosen = now;
        for (i=0; i<(int)sizeof(sunPassword); i++)
          sunPassword[i] = random() & 0xFF;
      }

      memcpy(&outbuf[0], (unsigned char[]){ 0x53, 0x41, 0x54, 0x4C, 0x01, 0x01 }, 6);
      memcpy(&outbuf[6], sunPassword, 8);
      *(unsigned short*)&outbuf[14] = htons(sunIntervalSec);
      *(unsigned short*)&outbuf[16] = htons(planetIntervalSec);
      *(in_addr_t*)&outbuf[18] = planet.sin_addr.s_addr;
      *(unsigned short*)&outbuf[22] = planet.sin_port;
      strcpy((char*)&outbuf[32], email);
      strcpy((char*)&outbuf[33+strlen(email)], identifier);
      memcpy(&outbuf[34+strlen(email)+strlen(identifier)], planetPassword, planetPasswordLen);
      totalBytesSent += 34+strlen(email)+strlen(identifier)+planetPasswordLen;
      *(unsigned int*)&outbuf[24] = htonl(totalBytesSent);
      *(unsigned int*)&outbuf[28] = htonl(totalBytesReceived);
      sendto(sock, &outbuf, 34+strlen(email)+strlen(identifier)+planetPasswordLen, 0, (struct sockaddr*)&sun, sizeof(sun));
    }

    struct pollfd fd = (struct pollfd){ fd:sock, events:POLLIN, 0 };
    int timeoutMillis = (int)max(1, min(nextPlanetReportMillis-now, nextSunReportMillis-now));
    int ret = poll(&fd, 1, timeoutMillis);

    if (ret < 0) {
      if ((errno == ECONNREFUSED) || (errno == ECONNRESET))
        continue;
      if (errno != EINTR) {
        fprintf(stderr, "Unkown error while listening (%s)", strerror(errno));
        return 4;
      }
    }

    if (ret == 1) {
      socklen_t addrLen = sizeof(srcaddr);
      int size = recvfrom(sock, &inbuf, sizeof(inbuf), 0, (struct sockaddr*)&srcaddr, &addrLen);
      struct timeval tv;
#ifdef __linux__
      ioctl(sock, SIOCGSTAMP, &tv);
#else
      gettimeofday(&tv, NULL);
#endif
      unsigned int recvTS = tv.tv_usec + (tv.tv_sec%4294)*1000000;

      if ((size>=24) && (srcaddr.sin_port == sun.sin_port) && (srcaddr.sin_addr.s_addr == sun.sin_addr.s_addr)) {
        if (!memcmp(&inbuf[6], sunPassword, 8)) {
          sunIntervalSec = max(1, ntohs(*(unsigned short*)&inbuf[14]));
          planetIntervalSec = max(1, ntohs(*(unsigned short*)&inbuf[16]));
          planet.sin_addr.s_addr = *(in_addr_t*)&inbuf[18];
          planet.sin_port = *(unsigned short*)&inbuf[22];
          planetPasswordLen = (size-24);
          memcpy(planetPassword, &inbuf[24], planetPasswordLen);
          nextPlanetReportMillis = nextSunReportMillis = 0;
          totalBytesReceived += 28+size;
        }
      }
      
      if ((size>=2) && planet.sin_addr.s_addr && (srcaddr.sin_port == planet.sin_port) && (srcaddr.sin_addr.s_addr == planet.sin_addr.s_addr)) {
        int rid = ntohs(*(unsigned short*)&inbuf[0]);
        int offset;
        totalBytesReceived += 28+size;
        
        for (offset=0; offset<=(size-4); offset+=4) {

          /* We read pairs (id,size), terminated by id=0 */

          int id = ntohs(*(unsigned short*)&inbuf[offset]);
          int responseSize = ntohs(*(unsigned short*)&inbuf[offset+2]);

          if (id == 0)
            break;

          /* 0xFFFE tells us to erase our history list, and to stop parsing */

          if (id == 0xFFFE) {
            for (i=0; i<PROBE_HISTORY_LENGTH; i++)
              probeHistory[i] = 0;

            probeIdsInHistory = 0;
            memcpy(outbuf, inbuf, min(size, (int)sizeof(outbuf)));
            break;
          }

          /* Have we seen this ID before? */

          char seenBefore = 0;
          for (i=0; i<PROBE_HISTORY_LENGTH; i++) {
            if (probeHistory[i] == id)
              seenBefore = 1;
          }

          /* Only respond if we haven't, or if the ID is 0xFFFF */

          if (!seenBefore || (id == 0xFFFF)) {
            if (responseSize >= 2)
              *(unsigned short*)&outbuf[0] = htons(rid);

            if (responseSize >= 4)
              *(unsigned short*)&outbuf[2] = htons(id);

            int numIdsToInclude = (responseSize-4)/2;
            if (numIdsToInclude > PROBE_HISTORY_MAX_INCLUDE)
              numIdsToInclude = PROBE_HISTORY_MAX_INCLUDE;
            if (numIdsToInclude > probeIdsInHistory)
              numIdsToInclude = probeIdsInHistory;

            for (i=0; i<numIdsToInclude; i++)
              *(unsigned short*)&outbuf[4+2*i] = ntohs(probeHistory[i]);

            *(unsigned short*)&outbuf[4+2*numIdsToInclude] = ntohs(0);
            *(unsigned int*)&outbuf[6+2*numIdsToInclude] = ntohl(recvTS);
            gettimeofday(&tv, NULL);
            *(unsigned int*)&outbuf[10+2*numIdsToInclude] = ntohl(tv.tv_usec + (tv.tv_sec%4294)*1000000);

            if (responseSize > 0) {
              sendto(sock, &outbuf, responseSize, 0, (struct sockaddr*)&planet, sizeof(planet));
              totalBytesSent += 28+responseSize;
            }

            if (id != 0xFFFF) {
              for (i=PROBE_HISTORY_LENGTH-2; i>=0; i--)
                probeHistory[i+1] = probeHistory[i];
              probeHistory[0] = id;
              if (probeIdsInHistory < PROBE_HISTORY_LENGTH)
                probeIdsInHistory ++;
            }
          }
        }
      }
    }
  }
}
