using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Text;
using System.Windows.Forms;
using System.Security.Cryptography;
using System.Timers;
using System.Runtime.InteropServices;
using Microsoft.Win32;

namespace Win32Satellite
{
    
	static class Program
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
        /// 

        const byte VERSION_MAJOR = 4;
        const byte VERSION_MINOR = 1;
        const string versionString = "Version 0.9.2 (p1)";
        const int PASSWORD_VALID_SECONDS = 120;
        const int PROBE_HISTORY_LENGTH = 100;
        const int PROBE_HISTORY_MAX_INCLUDE = 20;
        const int MAX_CODE_LENGTH = 100;

        static Socket serverSocket;
        static RNGCryptoServiceProvider rng;
        static IPEndPoint sunEndpoint;
        static Boolean[] settingsChanged;
        static Boolean[] enabledReport;
        static RegistryKey key;
        static SatelliteSettingsForm form;
        static IdentifierForm iform;
        static AboutBox aboutBox;
        static System.Timers.Timer sunReportTimer;
        static System.Timers.Timer planetReportTimer;

        static string identifier;
        static string email;
        static IPAddress planetIP;
        static int planetPort;
        static int sunReportIntervalSec;
        static int planetReportIntervalSec;
        static byte[] password;
        static int lastPasswordChange;
        static byte[] code;
        static int codeLength;
        static uint totalBytesSent;
        static uint totalBytesReceived;
        
        /* Keepalive packet to the planet (once per second; keeps the NAT open) */

        static void planetReport(object source, ElapsedEventArgs e)
        {
            byte[] data = new byte[1024];
            if (!enabledReport[0] || (planetIP == null) || (planetPort == 0))
                return;

            if (settingsChanged[0])
            {
                readSettings();
                settingsChanged[0] = false;
            }

            data[0] = 0xFF;
            data[1] = 0xFE;
            data[2] = 0x53;
            data[3] = 0x41;
            data[4] = 0x54;
            data[5] = 0x4C;

            data[6] = VERSION_MAJOR;
            data[7] = VERSION_MINOR;

            for (int i = 0; i < codeLength; i++)
                data[8 + i] = (byte)code[i];

            try
            {
                serverSocket.SendTo(data, 8 + codeLength, SocketFlags.None, new IPEndPoint(planetIP, planetPort));
                totalBytesSent += (uint)(28 + 8 + codeLength);
            }
            catch (SocketException)
            {
                // MessageBox.Show("Socket exception", "SatelliteLab", MessageBoxButtons.OK);
            }
        }

        static void putShortNetworkOrder(int val, byte[] data, int offset) 
        {
            byte[] pb = BitConverter.GetBytes(IPAddress.HostToNetworkOrder((short)val));
            data[offset] = pb[0];
            data[offset+1] = pb[1];
        }

        static void putIntNetworkOrder(uint val, byte[] data, int offset)
        {
            byte[] pb = BitConverter.GetBytes(IPAddress.HostToNetworkOrder(val));
            data[offset] = pb[4];
            data[offset + 1] = pb[5];
            data[offset + 2] = pb[6];
            data[offset + 3] = pb[7];
        }

        /* Keepalive packet to the sun (once every 59 seconds) */

        static void sunReport(object source, ElapsedEventArgs e)
        {
            byte[] data = new byte[1024];
            if (!enabledReport[0])
                return;

            if (settingsChanged[0] || (sunEndpoint == null))
            {
                readSettings();
                settingsChanged[0] = false;
            }

            if (sunEndpoint != null)
            {
                int now = (int)((DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds);
                if (now > (lastPasswordChange + PASSWORD_VALID_SECONDS))
                {
                    rng.GetBytes(password);
                    lastPasswordChange = now;
                }

                data[0] = 0x53;
                data[1] = 0x41;
                data[2] = 0x54;
                data[3] = 0x4C;

                data[4] = VERSION_MAJOR;
                data[5] = VERSION_MINOR;

                for (int i = 0; i < 8; i++)
                    data[6 + i] = password[i];

                putShortNetworkOrder(sunReportIntervalSec, data, 14);
                putShortNetworkOrder(planetReportIntervalSec, data, 16);
                if (planetIP != null)
                {
                    byte[] pb = planetIP.GetAddressBytes();
                    for (int i = 0; i < 4; i++)
                        data[18 + i] = pb[i];
                }
                else
                {
                    for (int i = 0; i < 4; i++)
                        data[18 + i] = 0;
                }
                putShortNetworkOrder(planetPort, data, 22);
                putIntNetworkOrder(totalBytesReceived, data, 28);

                int pos = 32;
                for (int i = 0; i < email.Length; i++)
                    data[pos + i] = (byte)email[i];

                data[pos + email.Length] = 0;
                pos += email.Length + 1;

                for (int i = 0; i < identifier.Length; i++)
                    data[pos + i] = (byte)identifier[i];

                data[pos + identifier.Length] = 0;
                pos += identifier.Length + 1;

                for (int i = 0; i < codeLength; i++)
                    data[pos + i] = code[i];

                pos += codeLength;

                totalBytesSent += (uint)(28 + pos);
                putIntNetworkOrder(totalBytesSent, data, 24);
                
                try
                {
                    serverSocket.SendTo(data, pos, SocketFlags.None, sunEndpoint);
                }
                catch (SocketException)
                {
                    //MessageBox.Show("Socket exception", "SatelliteLab", MessageBoxButtons.OK);
                }
            }
        }

        static void readSettings()
        {
            string ssunaddress = (string)key.GetValue("sunaddress");
            string ssunport = (string)key.GetValue("sunport");
            identifier = (string)key.GetValue("identifier");
            email = (string)key.GetValue("email");
            sunEndpoint = null;
            try
            {
                IPHostEntry entry = Dns.GetHostEntry(ssunaddress);
                IPAddress sunIP = entry.AddressList[0];
                int sunPort = Convert.ToInt32(ssunport);
                sunEndpoint = new IPEndPoint(sunIP, sunPort);
            }
            catch (SocketException)
            {
            }
        }

        [DllImport("Kernel32.dll")]
        private static extern bool QueryPerformanceCounter(
            out long lpPerformanceCount);

        [DllImport("Kernel32.dll")]
        private static extern bool QueryPerformanceFrequency(
            out long lpFrequency);

        static void satellite()
        {
            long pcFrequency;
            byte[] inbuf = new byte[65536];
            byte[] outbuf = new byte[65536];
            int[] probeHistory = new int[PROBE_HISTORY_LENGTH];
            int probeIdsInHistory = 0;

            QueryPerformanceFrequency(out pcFrequency);

            for (int i=0; i<PROBE_HISTORY_LENGTH; i++)
                probeHistory[i] = 0;

            /* Main loop */

            while (true)
            {
                if (settingsChanged[0])
                {
                    readSettings();
                    settingsChanged[0] = false;
                }

                try
                {
                    IPEndPoint sender = new IPEndPoint(IPAddress.Any, 0);
                    EndPoint Remote = (EndPoint)sender;

                    int recv = serverSocket.ReceiveFrom(inbuf, ref Remote);
                    long recvTS;
                    QueryPerformanceCounter(out recvTS);

                    if ((recv < 4) || (recv>65535))
                        continue;

                    /* If this is a packet from the sun, handle it */

                    sender = (IPEndPoint)Remote;
                    if ((sunEndpoint != null) && sender.Equals(sunEndpoint))
                    {
                        totalBytesReceived += (uint)(28 + recv);

                        if (recv >= 24)
                        {
                            bool passwordOK = true;
                            for (int i = 0; i < 8; i++)
                            {
                                if (inbuf[6 + i] != password[i])
                                    passwordOK = false;
                            }

                            if (passwordOK)
                            {
                                byte[] b = new byte[4];
                                bool nonzeroIP = false;
                                for (int i = 0; i < 4; i++)
                                {
                                    b[i] = inbuf[18 + i];
                                    if (b[i] != 0)
                                        nonzeroIP = true;
                                }
                                planetIP = nonzeroIP ? new IPAddress(b) : null;
                                planetPort = (int)((UInt16)IPAddress.NetworkToHostOrder(BitConverter.ToInt16(inbuf, 22)));
                                sunReportIntervalSec = (int)((UInt16)IPAddress.NetworkToHostOrder(BitConverter.ToInt16(inbuf, 14)));
                                planetReportIntervalSec = (int)((UInt16)IPAddress.NetworkToHostOrder(BitConverter.ToInt16(inbuf, 16)));
                                if (sunReportIntervalSec < 1)
                                    sunReportIntervalSec = 1;
                                if (sunReportIntervalSec > 3600)
                                    sunReportIntervalSec = 3600;
                                if (planetReportIntervalSec < 1)
                                    planetReportIntervalSec = 1;
                                if (planetReportIntervalSec > 3600)
                                    planetReportIntervalSec = 3600;

                                codeLength = recv - 24;
                                if (codeLength > MAX_CODE_LENGTH)
                                    codeLength = MAX_CODE_LENGTH;

                                for (int i = 0; i < codeLength; i++)
                                    code[i] = inbuf[24 + i];

                                sunReportTimer.Enabled = false;
                                sunReportTimer.Interval = sunReportIntervalSec * 1000;
                                sunReportTimer.Enabled = true;
                                planetReportTimer.Enabled = false;
                                planetReport(null, null);
                                planetReportTimer.Interval = planetReportIntervalSec * 1000;
                                planetReportTimer.Enabled = true;
                            }
                            else
                            {
                                // MessageBox.Show("Bad password");
                            }
                        }

                        continue;
                    }

                    if (!enabledReport[0])
                        continue;
                    if (sender.Port != planetPort)
                        continue;
                    if (!sender.Address.Equals(planetIP))
                        continue;

                    totalBytesReceived += (uint)(28 + recv);

                    int rid = (int)((UInt16)IPAddress.NetworkToHostOrder(BitConverter.ToInt16(inbuf, 0)));
                    for (int offset=0; offset<=(recv-4); offset+=4) {
    
                        /* We read pairs (id,size), terminated by id=0 */
    
                        int id = (int)((UInt16)IPAddress.NetworkToHostOrder(BitConverter.ToInt16(inbuf, offset)));
                        int responseSize = (int)((UInt16)IPAddress.NetworkToHostOrder(BitConverter.ToInt16(inbuf, offset+2)));
      
                        if (id == 0)
                            break;

                        /* 0xFFFE tells us to erase our history list, and to stop parsing */
        
                        if (id == 0xFFFE) {
                            for (int i=0; i<PROBE_HISTORY_LENGTH; i++)
                                probeHistory[i] = 0;

                            for (int i = 0; i < recv; i++)
                                outbuf[i] = inbuf[i];

                            probeIdsInHistory = 0;
                            break;
                        }

                        /* Have we seen this ID before? */

                        bool seenBefore = false;
                        for (int i=0; i<PROBE_HISTORY_LENGTH; i++) {
                            if (probeHistory[i] == id)
                                seenBefore = true;
                        }

                        /* Only respond if we haven't, or if the ID is 0xFFFF */

                        if (!seenBefore || (id == 0xFFFF)) {
                            if (responseSize >= 2)
                                putShortNetworkOrder(rid, outbuf, 0);

                            if (responseSize >= 4)
                                putShortNetworkOrder(id, outbuf, 2);

                            int numIdsToInclude = (responseSize-4)/2;
                            if (numIdsToInclude > PROBE_HISTORY_MAX_INCLUDE)
                                numIdsToInclude = PROBE_HISTORY_MAX_INCLUDE;
                            if (numIdsToInclude > probeIdsInHistory)
                                numIdsToInclude = probeIdsInHistory;

                            for (int i=0; i<numIdsToInclude; i++)
                                putShortNetworkOrder(probeHistory[i], outbuf, 4+2*i);

                            putShortNetworkOrder(0, outbuf, 4 + 2 * numIdsToInclude);

                            long nowTS;
                            QueryPerformanceCounter(out nowTS);
                            putIntNetworkOrder((uint)(recvTS*1000000.0/pcFrequency), outbuf, 6 + 2 * numIdsToInclude);
                            putIntNetworkOrder((uint)(nowTS*1000000.0/pcFrequency), outbuf, 10 + 2 * numIdsToInclude);

                            if (responseSize > 0) {
                                try
                                {
                                    serverSocket.SendTo(outbuf, responseSize, SocketFlags.None, new IPEndPoint(planetIP, planetPort));
                                    totalBytesSent += (uint)(28 + responseSize);
                                }
                                catch (SocketException)
                                {
                                    //MessageBox.Show("Socket exception", "SatelliteLab", MessageBoxButtons.OK);
                                }
                            }

                            if (id != 0xFFFF)
                            {
                                for (int i = PROBE_HISTORY_LENGTH - 2; i >= 0; i--)
                                    probeHistory[i + 1] = probeHistory[i];
                                probeHistory[0] = id;
                                if (probeIdsInHistory < PROBE_HISTORY_LENGTH)
                                    probeIdsInHistory++;
                            }
                        }
                    }
                }
                catch (SocketException)
                {
                    //MessageBox.Show("Socket Exc2", "SatelliteLab", MessageBoxButtons.OK);
                }
            }
        }

		[STAThread]
		static void Main()
		{
            bool needIdentifierDialog = false;
            key = Registry.CurrentUser.OpenSubKey("Software\\SatelliteLab", true);
            if (key == null)
                key = Registry.CurrentUser.CreateSubKey("Software\\SatelliteLab");

            if (key.GetValue("sunaddress") == null)
                key.SetValue("sunaddress", SatelliteLab.Properties.Resources.DefaultSunAddress);

            if (key.GetValue("sunport") == null)
                key.SetValue("sunport", SatelliteLab.Properties.Resources.DefaultSunPort);

            if (key.GetValue("identifier") == null)
            {
                needIdentifierDialog = true;
                key.SetValue("identifier", "desktop");
            }

            if (key.GetValue("email") == null)
            {
                needIdentifierDialog = true;
                key.SetValue("email", "user@domain.com");
            }
            else
            {
                string email = (string)key.GetValue("email");
                if (!email.Contains("@") || email.Equals("user@domain.com"))
                    needIdentifierDialog = true;
            }

            serverSocket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
            EndPoint ep = new IPEndPoint(IPAddress.Any, Convert.ToInt32(SatelliteLab.Properties.Resources.LocalPort));
            try
            {
                serverSocket.Bind(ep);
            }
            catch (SocketException)
            {
                MessageBox.Show("SatelliteLab cannot start because TCP port " + SatelliteLab.Properties.Resources.LocalPort +
                    " is already in use. Maybe another instance of SatelliteLab is already running on this machine?", "SatelliteLab error");
                System.Environment.Exit(0);
            }
            planetIP = null;
            planetPort = 0;
            sunReportIntervalSec = Convert.ToInt32(SatelliteLab.Properties.Resources.InitialSunReportIntervalSec);
            planetReportIntervalSec = Convert.ToInt32(SatelliteLab.Properties.Resources.InitialPlanetReportIntervalSec);
            password = new byte[8];
            code = new byte[MAX_CODE_LENGTH];
            codeLength = 0;
            totalBytesSent = 0;
            totalBytesReceived = 0;
            rng = new RNGCryptoServiceProvider();
            rng.GetBytes(password);
            lastPasswordChange = (int)((DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds);
            settingsChanged = new Boolean[] { false };
            enabledReport = new Boolean[] { true };
            readSettings();

            sunReportTimer = new System.Timers.Timer(sunReportIntervalSec * 1000);
            sunReportTimer.Elapsed += new ElapsedEventHandler(sunReport);
            sunReportTimer.Enabled = true;
            planetReportTimer = new System.Timers.Timer(planetReportIntervalSec * 1000);
            planetReportTimer.Elapsed += new ElapsedEventHandler(planetReport);
            planetReportTimer.Enabled = true;

            Application.EnableVisualStyles();
            aboutBox = new AboutBox(versionString);
            form = new SatelliteSettingsForm(key, settingsChanged, enabledReport, aboutBox);
            iform = new IdentifierForm(key, settingsChanged, enabledReport);
            if (needIdentifierDialog)
            {
                enabledReport[0] = false;
                iform.Show();
            }

            Thread t1 = new Thread(new ThreadStart(satellite));
            t1.Start();

            sunReport(null, null);

            Application.Run();
        }
	}
}
