using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using Microsoft.Win32;
using System.Windows.Forms;
using System.Threading;

namespace Win32Satellite
{
	public partial class SatelliteSettingsForm : Form
	{
		private bool satelliteEnabled;
        private RegistryKey key;
        private Boolean[] settingsChanged;
        private Boolean[] enabledReport;
        private AboutBox aboutBox;

		public SatelliteSettingsForm(RegistryKey regkey, Boolean[] settingsReport, Boolean[] enabledReportExt, AboutBox about)
		{
			InitializeComponent();
            key = regkey;
            settingsChanged = settingsReport;
            enabledReport = enabledReportExt;
            aboutBox = about;
		}

		private void SatelliteSettingsForm_Load(object sender, EventArgs e)
		{
            satelliteEnabled = true;
            enabledReport[0] = true;

            SatelliteEnabledCheckBox.Checked = satelliteEnabled;
            EnabledToolStripMenuItem.Checked = satelliteEnabled;
            SystemTrayIcon.Visible = true;
            ControllerTextBox.Text = (string)key.GetValue("sunaddress");
            PortTextBox.Text = (string)key.GetValue("sunport");
            EmailTextBox.Text = (string)key.GetValue("email");
            IdentifierTextBox.Text = (string)key.GetValue("identifier");
		}

        private void terminateApp()
        {
            SystemTrayIcon.Visible = false;
            System.Environment.Exit(0);
        }

		private void settingsOKButton_Click(object sender, EventArgs e)
		{
			this.Hide();

            key.SetValue("email", EmailTextBox.Text);
            key.SetValue("identifier", IdentifierTextBox.Text);
            key.SetValue("sunport", PortTextBox.Text);
            key.SetValue("sunaddress", ControllerTextBox.Text);
            settingsChanged[0] = true;

            satelliteEnabled = SatelliteEnabledCheckBox.Checked;
            enabledReport[0] = satelliteEnabled;
            EnabledToolStripMenuItem.Checked = satelliteEnabled;
        }

		private void settingsCancelButton_Click(object sender, EventArgs e)
		{
            terminateApp();
        }

		private void SettingsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			TrayContextMenuStrip.Hide();
			this.Show();
		}

		private void EnabledToolStripMenuItem_Click(object sender, EventArgs e)
		{
			// Toggle the checkbox on the menu item
			satelliteEnabled = ! EnabledToolStripMenuItem.Checked;
            enabledReport[0] = satelliteEnabled;
			EnabledToolStripMenuItem.Checked = satelliteEnabled;
			SatelliteEnabledCheckBox.Checked = satelliteEnabled;
			TrayContextMenuStrip.Hide();
		}

		private void ExitToolStripMenuItem_Click(object sender, EventArgs e)
		{
            terminateApp();
		}

		private void AlarmNotifyIcon_BalloonTipClicked(object sender, EventArgs e)
		{
            this.Show();
        }

		private void AlarmNotifyIcon_MouseDoubleClick(object sender, MouseEventArgs e)
		{
			this.Show();
		}

		private void AlarmNotifyIcon_MouseClick(object sender, MouseEventArgs e)
		{
			if( e.Button == MouseButtons.Right )
			{
	            TrayContextMenuStrip.Show();
		    }
		}

        private void Label2_Click(object sender, EventArgs e)
        {

        }

        private void MessageTextBox_TextChanged(object sender, EventArgs e)
        {

        }

        private void SoundBrowserDialog_FileOk(object sender, CancelEventArgs e)
        {

        }

        private void showAboutBox(object sender, EventArgs e)
        {
            aboutBox.Show();
        }
	}
}